

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>

#include "uart.h"

void uart_init(unsigned long baud){
	
	//From chapter 24.3 in datasheet
	PORTB.OUTSET |= (1 << PIN2_bp);
	PORTB.DIRSET |= (1 << PIN2_bp); 		//Setting up TX pin as high output
	USART0.BAUD = baud; 	//Set baudrate
	//USART.CTRLC CMODE bits default to: async, 1 stop bit, 8 bit character size
	USART0.CTRLB |= (1 << USART_RXEN_bp) | (1 << USART_TXEN_bp);	//Enable RX and TX
	
	
	USART0.CTRLA |= (1 << USART_RXCIE_bp); //Enable interupts on RX
	
	fdevopen(uart_transmit, NULL); //This allows us to use printf instead of writing a single character at a time
	//It also allows for easily printing variable values:  printf("X is currently: %d \n", x_var);
}



// function to transmit data
int uart_transmit(char data, FILE* stream){
	while (!(USART0.STATUS & (1 << USART_DREIF_bp))){
		//Wait for ongoing transmission to finish (if there is one)
	} 
	USART0.TXDATAL = data; //Put new data in register
	return 0;
}


//Interrupt service routine for receiving
ISR(USART0_RXC_vect){
	
	uint8_t data = USART0.RXDATAL; //We must read the data to clear the interrupt flag
	//Here we can do something with data if we would want that
	
}